@@ -9,6 +9,7 @@
  * license.
  */
 
+#include <math.h>
 #include <gdk/gdkkeysyms.h>
 
 #include "book-view.h"
@@ -49,6 +50,21 @@ typedef struct
 } PageView;
 
 
+typedef enum
+{
+    CROP_NONE = 0,
+    CROP_MIDDLE,
+    CROP_TOP,
+    CROP_BOTTOM,
+    CROP_LEFT,
+    CROP_RIGHT,
+    CROP_TOP_LEFT,
+    CROP_TOP_RIGHT,
+    CROP_BOTTOM_LEFT,
+    CROP_BOTTOM_RIGHT
+} CropLocation;
+
+
 struct BookViewPrivate
 {
     /* Book being rendered */
@@ -58,19 +74,17 @@ struct BookViewPrivate
     /* True if the view needs to be laid out again */
     gboolean need_layout;
 
-    /* Dimensions of area to render into (pixels) */
-    // FIXME: Can use widget->allocation.width?
-    gint width, height;
-
     /* Amount to offset view by (pixels) */
-    gint x_offset, y_offset;
+    gint x_offset;
 
     Page *selected_page;
-    
+
     /* The page the crop is being moved on or NULL */
     PageView *selected_crop;
+    CropLocation crop_location;
     gdouble selected_crop_px, selected_crop_py;
     gint selected_crop_x, selected_crop_y;
+    gint selected_crop_w, selected_crop_h;
 
     /* Widget being rendered to */
     GtkWidget *widget;
@@ -82,6 +96,10 @@ struct BookViewPrivate
 
     /* Last location of mouse */
     gdouble mouse_x, mouse_y;
+  
+    gboolean animate_empty_pages;
+    guint animate_timeout;
+    gint animate_n_segments, animate_segment;
 };
 
 G_DEFINE_TYPE (BookView, book_view, G_TYPE_OBJECT);
@@ -267,8 +285,6 @@ Book *book_view_get_book (BookView *view)
 static gboolean
 configure_cb (GtkWidget *widget, GdkEventConfigure *event, BookView *view)
 {
-    view->priv->width = event->width;
-    view->priv->height = event->height;
     view->priv->need_layout = TRUE;
     return FALSE;
 }
@@ -344,14 +360,45 @@ render_page (BookView *view, PageView *page, cairo_t *context)
     cairo_set_line_width (context, 1);
 
     /* Draw background */
-    cairo_translate (context, view->priv->x_offset + page->x, view->priv->y_offset + page->y);
-    cairo_translate (context, 1, 1);
+    cairo_translate (context, page->x - view->priv->x_offset, page->y);
+    cairo_translate (context, page->border, page->border);
     gdk_cairo_set_source_pixbuf (context, page->image, 0, 0);
     cairo_paint (context);
 
-    /* Draw scan line */
     scan_line = page_get_scan_line (page->page);
-    if (scan_line >= 0) {
+  
+    if (scan_line == 0) {
+        gdouble inner_radius, outer_radius, x, y, arc, offset = 0.0;
+        gint i;
+
+        if (page->width > page->height)
+            outer_radius = 0.15 * page->width;
+        else
+            outer_radius = 0.15 * page->height;
+        arc = M_PI / view->priv->animate_n_segments;
+
+        /* Space circles */
+        x = outer_radius * sin (arc);
+        y = outer_radius * (cos (arc) - 1.0);
+        inner_radius = 0.6 * sqrt (x*x + y*y);
+
+        for (i = 0; i < view->priv->animate_n_segments; i++, offset += arc * 2) {
+            x = page->width / 2 + outer_radius * sin (offset);
+            y = page->height / 2 - outer_radius * cos (offset);
+            cairo_arc (context, x, y, inner_radius, 0, 2 * M_PI);
+
+            if (i == view->priv->animate_segment) {
+                cairo_set_source_rgb (context, 0.75, 0.75, 0.75);
+                cairo_fill_preserve (context);
+            }
+
+            cairo_set_source_rgb (context, 0.5, 0.5, 0.5);          
+            cairo_stroke (context);
+        }
+    }
+
+    /* Draw scan line */
+    if (scan_line > 0) {
         double s;
         double x1, y1, x2, y2;
         
@@ -411,9 +458,6 @@ render_page (BookView *view, PageView *page, cairo_t *context)
     }
 
     /* Draw page border */
-    /* NOTE: Border width and height is rounded up so border is sharp.  Background may not
-     * extend to border, should fill with white (?) before drawing scanned image or extend
-     * edges slightly */
     if (page->page == view->priv->selected_page) {
         if (gtk_widget_has_focus (view->priv->widget))
             cairo_set_source_rgb (context, 1, 0, 0);
@@ -432,20 +476,16 @@ render_page (BookView *view, PageView *page, cairo_t *context)
 
 
 static void
-layout (BookView *view)
+layout_into (BookView *view, gint width, gint height, gint *book_width, gint *book_height)
 {
-    gint i, n_pages;
-    gdouble max_width = 0, max_height = 0;
-    gdouble inner_width, inner_height;
+    gint inner_width, inner_height;
+    gint border = 1, spacing = 12;
+    gint max_width = 0, max_height = 0;
     gdouble max_aspect, inner_aspect;
-    gdouble border = 1, spacing = 12;
-    gdouble book_width = 0, book_height = 0;
-    gdouble x_offset = 0, y_offset = 0, scale;
-    gdouble x_range = 0, y_range = 0;
+    gdouble scale;
+    gint x_offset = 0;
+    gint i, n_pages;
 
-    if (!view->priv->need_layout)
-        return;
-    
     n_pages = book_get_n_pages (view->priv->book);
 
     /* Get area required to fit all pages */
@@ -456,10 +496,10 @@ layout (BookView *view)
         if (page_get_height (page) > max_height)
             max_height = page_get_height (page);
     }
-    
+
     /* Make space for fixed size border */
-    inner_width = view->priv->width - 2*border;
-    inner_height = view->priv->height - 2*border;
+    inner_width = width - 2*border;
+    inner_height = height - 2*border;
     
     /* Make space to see adjacent pages */
     // FIXME: If selected first or last only need half amount
@@ -471,67 +511,36 @@ layout (BookView *view)
     
     /* Scale based on width... */
     if (max_aspect > inner_aspect) {
-        scale = inner_width / max_width;
+        scale = (double)inner_width / max_width;
     }
     /* ...or height */
     else {
-        scale = inner_height / max_height;
+        scale = (double)inner_height / max_height;
     }
     
     /* Don't scale past exact resolution */
     if (scale >= 1.0)
         scale = 1.0;
-    
+   
     /* Get total dimensions of all pages */
+    *book_width = 0;
+    *book_height = 0;
     for (i = 0; i < n_pages; i++) {
         Page *p = book_get_page (view->priv->book, i);
         PageView *page = g_hash_table_lookup (view->priv->page_data, p);
         gdouble h;
 
         page_set_scale (page, scale);
-        update_page_view (page);
+        update_page_view (page); // FIXME: Don't scale image until next render
 
         h = page->height + (2 * border);
-        if (h > book_height)
-            book_height = h;
-        book_width += page->width + (2 * border);
+        if (h > *book_height)
+            *book_height = h;
+        *book_width += page->width + (2 * border);
         if (i != 0)
-            book_width += spacing;
+            *book_width += spacing;
     }
-    
-    /* Offset so pages are in the middle */
-    if (view->priv->width >= book_width) {
-        x_offset = (int) ((view->priv->width - book_width) / 2);
-        x_range = 0;
-        gtk_widget_hide (view->priv->scroll);
-    }
-    else {
-        x_offset = 0;
-        x_range = book_width - view->priv->width;
-        gtk_adjustment_set_upper (view->priv->adjustment, book_width);
-        gtk_adjustment_set_page_size (view->priv->adjustment, view->priv->width);
-        gtk_widget_show (view->priv->scroll);
-    }
-   
-    if (view->priv->height >= book_height) {
-        y_offset = (int) ((view->priv->height - book_height) / 2);
-        y_range = 0;
-    } else {
-        y_offset = 0;
-        y_range = (int) (book_height - view->priv->height);
-    }
-    
-    /* Clamp offsets */
-    if (view->priv->x_offset < -x_range)
-        view->priv->x_offset = -x_range;
-    if (view->priv->x_offset > 0)
-        view->priv->x_offset = 0;
-    if (view->priv->y_offset < -y_range)
-        view->priv->y_offset = -y_range;
-    if (view->priv->y_offset > 0)
-        view->priv->y_offset = 0;
-    gtk_adjustment_set_value (view->priv->adjustment, view->priv->x_offset);
-    
+
     for (i = 0; i < n_pages; i++) {
         Page *p = book_get_page (view->priv->book, i);
         PageView *page = g_hash_table_lookup (view->priv->page_data, p);
@@ -539,19 +548,118 @@ layout (BookView *view)
         page->border = border;
         page->x = x_offset;
         x_offset += page->width + (2 * page->border) + spacing;
-        page->y = y_offset + (book_height - page->height) / 2 - page->border;
+        page->y = (*book_height - page->height) / 2 - page->border;
+    }
+}
+
+
+static void
+layout (BookView *view)
+{
+    gint width, height, book_width, book_height;
+
+    if (!view->priv->need_layout)
+        return;
+  
+    /* Try and fit without scrollbar */
+    width = view->priv->widget->allocation.width;
+    height = view->priv->box->allocation.height;
+    layout_into (view, width, height, &book_width, &book_height);
+
+    /* Relayout with scrollbar */
+    if (book_width > view->priv->widget->allocation.width) {
+        /* Re-layout leaving space for scrollbar */
+        height = view->priv->widget->allocation.height;
+        layout_into (view, width, height, &book_width, &book_height);
+
+        /* Set scrollbar limits */
+        gtk_adjustment_set_upper (view->priv->adjustment, book_width);
+        gtk_adjustment_set_page_size (view->priv->adjustment, view->priv->widget->allocation.width);
+
+        /* Ensure that selected page is visible */
+        if (view->priv->selected_page) {
+            PageView *page;
+            gint left_edge, right_edge;
+
+            page = g_hash_table_lookup (view->priv->page_data, view->priv->selected_page);
+            left_edge = page->x;
+            right_edge = page->x + page->width + page->border * 2;
+
+            if (left_edge - view->priv->x_offset < 0) {
+                view->priv->x_offset = left_edge;
+            }
+            else if (right_edge - view->priv->x_offset > view->priv->widget->allocation.width) {
+                view->priv->x_offset = right_edge - view->priv->widget->allocation.width;
+            }
+        }
+
+        /* Make sure offset is still visible */
+        if (view->priv->x_offset < 0)
+            view->priv->x_offset = 0;
+        if (view->priv->x_offset > book_width - view->priv->widget->allocation.width)
+            view->priv->x_offset = book_width - view->priv->widget->allocation.width;
+
+        gtk_adjustment_set_value (view->priv->adjustment, view->priv->x_offset);
+        gtk_widget_show (view->priv->scroll);
+    } else {
+        view->priv->x_offset = (book_width - view->priv->widget->allocation.width) / 2;
+        gtk_widget_hide (view->priv->scroll);
     }
 
     view->priv->need_layout = FALSE;
 }
 
 
+static gboolean
+animation_cb (BookView *view)
+{
+    view->priv->animate_segment = (view->priv->animate_segment + 1) % view->priv->animate_n_segments;
+    gtk_widget_queue_draw (view->priv->widget);
+    return TRUE;
+}
+
+
+static void
+update_animation (BookView *view)
+{
+    gboolean animate = FALSE, is_animating;
+    gint i;
+  
+    for (i = 0; i < book_get_n_pages (view->priv->book); i++) {
+        Page *page;
+        page = book_get_page (view->priv->book, i);
+        if (page_get_scan_line (page) == 0) {
+            animate = TRUE;
+            break;
+        }
+    }
+  
+    is_animating = view->priv->animate_timeout != 0;
+    if (animate == is_animating)
+        return;
+  
+    if (animate) {
+        view->priv->animate_n_segments = 7;
+        view->priv->animate_segment = 0;
+        if (view->priv->animate_timeout == 0)
+            view->priv->animate_timeout = g_timeout_add (150, (GSourceFunc) animation_cb, view);
+    }
+    else
+    {
+        if (view->priv->animate_timeout != 0)
+            g_source_remove (view->priv->animate_timeout);
+        view->priv->animate_timeout = 0;
+    }
+}
+
+
 static gboolean
 expose_cb (GtkWidget *widget, GdkEventExpose *event, BookView *view)
 {
     gint i, n_pages;
-
     cairo_t *context;
+  
+    update_animation (view);
 
     n_pages = book_get_n_pages (view->priv->book);
     if (n_pages == 0)
@@ -574,21 +682,66 @@ expose_cb (GtkWidget *widget, GdkEventExpose *event, BookView *view)
 }
 
 
-static gboolean
-is_inside_crop (PageView *page, gint x, gint y)
+static CropLocation
+get_crop_location (PageView *page, gint x, gint y)
 {
     gint cx, cy, cw, ch;
     gint dx, dy, dw, dh;
+    gint ix, iy;
+    gint crop_border = 20;
+    gchar *name;
 
     if (!page_has_crop (page->page))
-        return FALSE;
+        return 0;
 
     page_get_crop (page->page, &cx, &cy, &cw, &ch);
     dx = page_to_screen_x (page, cx) + page->x;
     dy = page_to_screen_y (page, cy) + page->y;
     dw = page_to_screen_x (page, cw);
     dh = page_to_screen_y (page, ch);
-    return (x >= dx && x <= dx + dw) && (y >= dy && y <= dy + dh);
+    ix = x - dx;
+    iy = y - dy;
+
+    if (ix < 0 || ix > dw || iy < 0 || iy > dh)
+        return CROP_NONE;
+
+    /* Can't resize named crops */
+    name = page_get_named_crop (page->page);
+    if (name != NULL) {
+        g_free (name);
+        return CROP_MIDDLE;
+    }
+
+    // FIXME: Adjust edges when small
+
+    /* Top left */
+    if (ix < crop_border && iy < crop_border)
+        return CROP_TOP_LEFT;
+    /* Top right */
+    if (ix > dw - crop_border && iy < crop_border)
+        return CROP_TOP_RIGHT;
+    /* Bottom left */
+    if (ix < crop_border && iy > dh - crop_border)
+        return CROP_BOTTOM_LEFT;
+    /* Bottom right */
+    if (ix > dw - crop_border && iy > dh - crop_border)
+        return CROP_BOTTOM_RIGHT;
+
+    /* Left */
+    if (ix < crop_border)
+        return CROP_LEFT;
+    /* Right */
+    if (ix > dw - crop_border)
+        return CROP_RIGHT;
+    /* Top */
+    if (iy < crop_border)
+        return CROP_TOP;
+    /* Bottom */
+    if (iy > dh - crop_border)
+        return CROP_BOTTOM;
+
+    /* In the middle */
+    return CROP_MIDDLE;
 }
 
 
@@ -616,25 +769,30 @@ button_cb (GtkWidget *widget, GdkEventButton *event, BookView *view)
         gint x, y, left, right, top, bottom;
         
         update_page_view (page);
-        x = event->x - view->priv->x_offset;
-        y = event->y - view->priv->y_offset;
+        x = event->x + view->priv->x_offset;
+        y = event->y;
         left = page->x;
         right = page->x + page->width;
         top = page->y;
         bottom = page->y + page->height;
         if (x >= left && x <= right && y >= top && y <= bottom) {
+            CropLocation location;
+
             book_view_select_page (view, page->page);
             on_page = TRUE;
 
             /* See if selecting crop */
-            if (is_inside_crop (page, x, y)) {
+            location = get_crop_location (page, x, y);;
+            if (location != CROP_NONE) {
                 view->priv->selected_crop = page;
+                view->priv->crop_location = location;
                 view->priv->selected_crop_px = event->x;
                 view->priv->selected_crop_py = event->y;
                 page_get_crop (page->page,
                                &view->priv->selected_crop_x,
                                &view->priv->selected_crop_y,
-                               NULL, NULL);
+                               &view->priv->selected_crop_w,
+                               &view->priv->selected_crop_h);
             }
 
             break;
@@ -659,7 +817,7 @@ motion_cb (GtkWidget *widget, GdkEventMotion *event, BookView *view)
     gint x, y;
     gint sx, sy;
     gint i, n_pages;
-    gboolean over_crop = FALSE;
+    gint cursor = GDK_ARROW;
 
     sx = event->x - view->priv->mouse_x;
     sy = event->y - view->priv->mouse_y;
@@ -667,32 +825,140 @@ motion_cb (GtkWidget *widget, GdkEventMotion *event, BookView *view)
     view->priv->mouse_y = event->y;
 
     /* Location of cursor in book */
-    x = event->x - view->priv->x_offset;
-    y = event->y - view->priv->y_offset;
+    x = event->x + view->priv->x_offset;
+    y = event->y;
     
-    /* Check if inside crop */
+    /* Get cursor to use */
     n_pages = book_get_n_pages (view->priv->book);
     for (i = 0; i < n_pages; i++) {
         Page *p = book_get_page (view->priv->book, i);
         PageView *page = g_hash_table_lookup (view->priv->page_data, p);
+        CropLocation location;
 
-        if (is_inside_crop (page, x, y)) {
-            over_crop = TRUE;
-            continue;
+        location = get_crop_location (page, x, y);
+        switch (location) {
+        case CROP_MIDDLE:
+            cursor = GDK_HAND1;
+            break;
+        case CROP_TOP:
+            cursor = GDK_TOP_SIDE;
+            break;
+        case CROP_BOTTOM:
+            cursor = GDK_BOTTOM_SIDE;
+            break;
+        case CROP_LEFT:
+            cursor = GDK_LEFT_SIDE;
+            break;
+        case CROP_RIGHT:
+            cursor = GDK_RIGHT_SIDE;
+            break;
+        case CROP_TOP_LEFT:
+            cursor = GDK_TOP_LEFT_CORNER;
+            break;
+        case CROP_TOP_RIGHT:
+            cursor = GDK_TOP_RIGHT_CORNER;
+            break;
+        case CROP_BOTTOM_LEFT:
+            cursor = GDK_BOTTOM_LEFT_CORNER;
+            break;
+        case CROP_BOTTOM_RIGHT:
+            cursor = GDK_BOTTOM_RIGHT_CORNER;
+            break;
+        default:
+            break;
         }
     }
 
+    // FIXME: Do this in the rendering loop (what if disable/change crop without rotating) */
+    if (!view->priv->selected_crop)
+        set_cursor (view, cursor);
+
     /* Move the crop */
     if (view->priv->selected_crop && event->state & GDK_BUTTON1_MASK) {
         gint pw, ph;
-        gint cx, cy, cw, ch;
-        gint new_x, new_y;
+        gint cx, cy, cw, ch, dx, dy;
+        gint new_x, new_y, new_w, new_h;
         
         pw = page_get_width (view->priv->selected_crop->page);
         ph = page_get_height (view->priv->selected_crop->page);
         page_get_crop (view->priv->selected_crop->page, &cx, &cy, &cw, &ch);
-        new_x = view->priv->selected_crop_x + screen_to_page_x (view->priv->selected_crop, event->x - view->priv->selected_crop_px);
-        new_y = view->priv->selected_crop_y + screen_to_page_y (view->priv->selected_crop, event->y - view->priv->selected_crop_py);
+
+        dx = screen_to_page_x (view->priv->selected_crop, event->x - view->priv->selected_crop_px);
+        dy = screen_to_page_y (view->priv->selected_crop, event->y - view->priv->selected_crop_py);
+
+        new_x = view->priv->selected_crop_x;
+        new_y = view->priv->selected_crop_y;
+        new_w = view->priv->selected_crop_w;
+        new_h = view->priv->selected_crop_h;
+      
+        if (view->priv->crop_location == CROP_TOP_LEFT ||
+            view->priv->crop_location == CROP_LEFT ||
+            view->priv->crop_location == CROP_BOTTOM_LEFT) {
+            if (dx > new_w + 1)
+                dx = new_w + 1;
+            if (new_x + dx < 0)
+                dx = -new_x;
+        }
+        if (view->priv->crop_location == CROP_TOP_LEFT ||
+            view->priv->crop_location == CROP_TOP ||
+            view->priv->crop_location == CROP_TOP_RIGHT) {
+            if (dy > new_h + 1)
+                dy = new_h + 1;
+            if (new_y + dy < 0)
+                dy = -new_y;
+        }
+     
+        if (view->priv->crop_location == CROP_TOP_RIGHT ||
+            view->priv->crop_location == CROP_RIGHT ||
+            view->priv->crop_location == CROP_BOTTOM_RIGHT) {
+            if (new_w - dx < 1)
+                dx = new_w - 1;
+            if (new_x + new_w + dx > pw)
+                dx = pw - new_x - new_w;
+        }
+        if (view->priv->crop_location == CROP_BOTTOM_LEFT ||
+            view->priv->crop_location == CROP_BOTTOM ||
+            view->priv->crop_location == CROP_BOTTOM_RIGHT) {
+            if (new_h - dy < 1)
+                dy = new_h - 1;
+            if (new_y + new_h + dy > ph)
+                dy = ph - new_y - new_h;
+        }
+
+        if (view->priv->crop_location == CROP_MIDDLE) {
+            new_x += dx;
+            new_y += dy;          
+        }
+        if (view->priv->crop_location == CROP_TOP_LEFT ||
+            view->priv->crop_location == CROP_LEFT ||
+            view->priv->crop_location == CROP_BOTTOM_LEFT) 
+        {
+            new_x += dx;
+            new_w -= dx;
+        }
+        if (view->priv->crop_location == CROP_TOP_LEFT ||
+            view->priv->crop_location == CROP_TOP ||
+            view->priv->crop_location == CROP_TOP_RIGHT) {
+            new_y += dy;
+            new_h -= dy;
+        }
+     
+        if (view->priv->crop_location == CROP_TOP_RIGHT ||
+            view->priv->crop_location == CROP_RIGHT ||
+            view->priv->crop_location == CROP_BOTTOM_RIGHT) {
+            new_w += dx;
+        }
+        if (view->priv->crop_location == CROP_BOTTOM_LEFT ||
+            view->priv->crop_location == CROP_BOTTOM ||
+            view->priv->crop_location == CROP_BOTTOM_RIGHT) {
+            new_h += dy;
+        }
+
+        if (new_w < 1)
+            new_w = 1;
+        if (new_h < 1)
+            new_h = 1;
+
         if (new_x > pw - cw)
             new_x = pw - cw;
         if (new_x < 0)
@@ -703,14 +969,10 @@ motion_cb (GtkWidget *widget, GdkEventMotion *event, BookView *view)
             new_y = 0;
 
         page_move_crop (view->priv->selected_crop->page, new_x, new_y);
+        if (new_w != cw || new_h != ch)
+            page_set_custom_crop (view->priv->selected_crop->page, new_w, new_h);
     }
-    
-    // FIXME: Do this in the rendering loop (what if disable/change crop without rotating) */
-    if (view->priv->selected_crop || over_crop)
-        set_cursor (view, GDK_HAND1);
-    else
-        set_cursor (view, GDK_ARROW);
-   
+
     return FALSE;
 }
 
@@ -749,7 +1011,7 @@ focus_cb (GtkWidget *widget, GdkEventFocus *event, BookView *view)
 static void
 scroll_cb (GtkAdjustment *adjustment, BookView *view)
 {
-   view->priv->x_offset = - ((int) (gtk_adjustment_get_value (view->priv->adjustment) + 0.5));
+   view->priv->x_offset = (int) (gtk_adjustment_get_value (view->priv->adjustment) + 0.5);
    gtk_widget_queue_draw (view->priv->widget);
 }
 
@@ -785,25 +1047,9 @@ book_view_select_page (BookView *view, Page *page)
 
     view->priv->selected_page = page;
 
-    /* Make selected page visible */
+    /* Re-layout to make selected page visible */
     if (view->priv->selected_page) {
-        PageView *page;
-        gint left_edge, right_edge;
-
-        page = g_hash_table_lookup (view->priv->page_data, view->priv->selected_page);
-
-        /* Left and right edges of the page */
-        left_edge = page->x;
-        right_edge = left_edge + page->border + page->width + page->border;
-
-        /* Make sure can see adjacent pages */
-        // FIXME: Hardcoded spacing 24
-
-        if (right_edge + view->priv->x_offset > view->priv->width)
-            view->priv->x_offset = view->priv->width - right_edge - 24;
-        else if (left_edge + view->priv->x_offset < 0)
-            view->priv->x_offset = -left_edge + 24;
-        gtk_adjustment_set_value (view->priv->adjustment, -view->priv->x_offset);
+        view->priv->need_layout = TRUE;      
     }
 
     gtk_widget_queue_draw (view->priv->widget);    
