@@ -30,8 +30,6 @@ static Book *book;
 
 static gboolean scanning = FALSE;
 
-static gboolean clear_pages = FALSE;
-
 static gboolean first_autodetect = TRUE;
 
 
@@ -60,7 +58,8 @@ update_scan_devices_cb (Scanner *scanner, GList *devices)
                                /* Warning displayed when no scanners are detected */
                                _("No scanners detected"),
                                /* Hint to user on why there are no scanners detected */
-                               _("Please check your scanner is connected and powered on"));
+                               _("Please check your scanner is connected and powered on"),
+			       FALSE);
             g_free (selected);
         }
     }
@@ -91,44 +90,13 @@ static void
 scanner_page_info_cb (Scanner *scanner, ScanPageInfo *info)
 {
     Page *page;
-    Orientation orientation = TOP_TO_BOTTOM;
-    gboolean do_crop = FALSE;
-    gchar *named_crop = NULL;
-    gint cx, cy, cw, ch;
 
     g_debug ("Page is %d pixels wide, %d pixels high, %d bits per pixel",
              info->width, info->height, info->depth);
-    
+
     page = book_get_page (book, -1);
-    if (page) {
-        orientation = page_get_orientation (page);
-        
-        do_crop = page_has_crop (page);
-        if (do_crop) {
-            named_crop = page_get_named_crop (page);
-            page_get_crop (page, &cx, &cy, &cw, &ch);
-        }
-    }
-
-    if (clear_pages) {
-        book_clear (book);
-        clear_pages = FALSE;
-    }
-
-    page = book_append_page (book, info->width, info->height, info->dpi, orientation);
-    if (do_crop) {
-        if (named_crop)  {
-            page_set_named_crop (page, named_crop);
-            g_free (named_crop);
-        }
-        else
-            page_set_custom_crop (page, cw, ch);
-        page_move_crop (page, cx, cy);
-    }
-    
-    ui_set_selected_page (ui, page);
-
-    page_start (page);
+    g_return_if_fail (page != NULL);
+    page_set_scan_area (page, info->width, info->height, info->dpi);
 }
 
 
@@ -159,7 +127,8 @@ scanner_failed_cb (Scanner *scanner, GError *error)
     ui_show_error (ui,
                    /* Title of error dialog when scan failed */
                    _("Failed to scan"),
-                   error->message);
+                   error->message,
+		   TRUE);
 }
 
 
@@ -178,14 +147,19 @@ scan_cb (SimpleScan *ui, const gchar *device, const gchar *profile_name, gboolea
           _("Scanned Document.pdf") },
         { "photo", 400, SCAN_MODE_COLOR,
           /* Default name for JPEG documents */
-          _("Scanned Document.jpeg") },
+          _("Scanned Document.jpg") },
         { "raw", 800, SCAN_MODE_COLOR,
           /* Default name for PNG documents */
           _("Scanned Document.png") },
         { NULL, 75, SCAN_MODE_COLOR,
           /* Default name for JPEG documents */
-          _("Scanned Document.jpeg") }                
+          _("Scanned Document.jpg") }                
     };
+    Page *page;
+    Orientation orientation = TOP_TO_BOTTOM;
+    gboolean do_crop = FALSE;
+    gchar *named_crop = NULL;
+    gint width = 100, height = 100, dpi = 100, cx, cy, cw, ch;
     gint i;
 
     g_debug ("Requesting scan of type %s from device '%s'", profile_name, device);
@@ -197,8 +171,39 @@ scan_cb (SimpleScan *ui, const gchar *device, const gchar *profile_name, gboolea
     /* Find this profile */
     for (i = 0; profiles[i].name && strcmp (profiles[i].name, profile_name) != 0; i++);
 
+    /* Copy info from previous page */
+    page = book_get_page (book, -1);
+    if (page) {
+        orientation = page_get_orientation (page);
+        width = page_get_width (page);
+        height = page_get_height (page);
+        dpi = page_get_dpi (page);
+        
+        do_crop = page_has_crop (page);
+        if (do_crop) {
+            named_crop = page_get_named_crop (page);
+            page_get_crop (page, &cx, &cy, &cw, &ch);
+        }
+    }  
+
     if (replace)
-        clear_pages = TRUE;
+        book_clear (book);
+
+    page = book_append_page (book, width, height, dpi, orientation);
+    if (do_crop) {
+        if (named_crop)  {
+            page_set_named_crop (page, named_crop);
+            g_free (named_crop);
+        }
+        else
+            page_set_custom_crop (page, cw, ch);
+        page_move_crop (page, cx, cy);
+    }
+    
+    ui_set_selected_page (ui, page);
+
+    page_start (page);
+  
     ui_set_default_file_name (ui, profiles[i].file_name);
     scanner_scan (scanner, device, NULL, profiles[i].dpi, profiles[i].mode, 8, continuous);
     //scanner_scan (scanner, device, "Flatbed", 50, mode, 8, continuous);
@@ -213,29 +218,6 @@ cancel_cb (SimpleScan *ui)
 }
 
 
-static void
-add_default_page ()
-{
-    if (book_get_n_pages (book) > 0)
-        return;
-
-    /* Start with A4 white image at 72dpi */
-    /* TODO: Should be like the last scanned image for the selected scanner */
-    book_append_page (book, 595, 842, 72, TOP_TO_BOTTOM);
-
-    /* Remove this page on the next scan */
-    clear_pages = TRUE;
-}
-
-
-static void
-page_removed_cb (Book *book, Page *page, SimpleScan *ui)
-{
-    /* Ensure always one page */
-    add_default_page ();
-}
-
-
 static gboolean
 save_book (const gchar *uri, GError **error)
 {
@@ -270,7 +252,8 @@ save_cb (SimpleScan *ui, const gchar *uri)
         ui_show_error (ui,
                        /* Title of error dialog when save failed */
                        _("Failed to save file"),
-                       error->message);
+                       error->message,
+		       FALSE);
         g_error_free (error);
     }
 }
@@ -441,17 +424,17 @@ get_options (int argc, char **argv)
 int
 main(int argc, char **argv)
 {
+    bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
+    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+    textdomain (GETTEXT_PACKAGE);
+   
     g_thread_init (NULL);
     gtk_init (&argc, &argv);
 
     get_options (argc, argv);
 
-    book = book_new ();
-    g_signal_connect (book, "page-removed", G_CALLBACK (page_removed_cb), NULL);
-    add_default_page ();
-
     ui = ui_new ();
-    ui_set_book (ui, book);
+    book = ui_get_book (ui);
     g_signal_connect (ui, "start-scan", G_CALLBACK (scan_cb), NULL);
     g_signal_connect (ui, "stop-scan", G_CALLBACK (cancel_cb), NULL);
     g_signal_connect (ui, "save", G_CALLBACK (save_cb), NULL);
