@@ -39,22 +39,46 @@ struct SimpleScanPrivate
 
     GtkWidget *window;
     GtkWidget *scan_button_label, *continuous_scan_button_label, *page_label;
-    GtkWidget *actions_box;
     GtkWidget *device_combo, *mode_combo;
     GtkTreeModel *device_model, *mode_model;
-    GtkWidget *replace_pages_check;
-    GtkWidget *preview_area;
-    GtkWidget *zoom_scale;
+    GtkWidget *preview_box, *preview_area, *preview_scroll;
     GtkWidget *page_delete_menuitem, *crop_rotate_menuitem;
+
+    GtkWidget *authorize_dialog;
+    GtkWidget *authorize_label;
+    GtkWidget *username_entry, *password_entry;
+
+    GtkWidget *preferences_dialog;
+    GtkWidget *replace_pages_check;
+
+    Book *book;
     BookView *book_view;
     gboolean updating_page_menu;
+    Orientation default_orientation;
+    gboolean book_is_placeholder; // FIXME: Needs to be cleared when scan starts
 
     gchar *default_file_name;
     gboolean scanning;
+
+    gint window_width, window_height;
+    gboolean window_is_maximized;
 };
 
 G_DEFINE_TYPE (SimpleScan, ui, G_TYPE_OBJECT);
 
+static struct
+{
+   const gchar *key;
+   Orientation orientation;
+} orientation_keys[] = 
+{
+  { "top-to-bottom", TOP_TO_BOTTOM },
+  { "bottom-to-top", BOTTOM_TO_TOP },
+  { "left-to-right", LEFT_TO_RIGHT },
+  { "right-to-left", RIGHT_TO_LEFT },
+  { NULL, 0 }
+};
+
 
 static gboolean
 find_scan_device (SimpleScan *ui, const char *device, GtkTreeIter *iter)
@@ -83,6 +107,31 @@ ui_set_default_file_name (SimpleScan *ui, const gchar *default_file_name)
 }
 
 
+void
+ui_authorize (SimpleScan *ui, const gchar *resource, gchar **username, gchar **password)
+{
+    GString *description;
+
+    description = g_string_new ("");
+    g_string_printf (description,
+                     /* Label in authorization dialog.  '%s' is replaced with the name of the resource requesting authorization */
+                     _("Username and password required to access '%s'"),
+                     resource);
+
+    gtk_entry_set_text (GTK_ENTRY (ui->priv->username_entry), *username ? *username : "");
+    gtk_entry_set_text (GTK_ENTRY (ui->priv->password_entry), "");
+    gtk_label_set_text (GTK_LABEL (ui->priv->authorize_label), description->str);
+    g_string_free (description, TRUE);
+
+    gtk_widget_show (ui->priv->authorize_dialog);
+    gtk_dialog_run (GTK_DIALOG (ui->priv->authorize_dialog));
+    gtk_widget_hide (ui->priv->authorize_dialog);
+
+    *username = g_strdup (gtk_entry_get_text (GTK_ENTRY (ui->priv->username_entry)));
+    *password = g_strdup (gtk_entry_get_text (GTK_ENTRY (ui->priv->password_entry)));
+}
+
+
 void
 ui_mark_devices_undetected (SimpleScan *ui)
 {
@@ -144,6 +193,31 @@ ui_set_selected_device (SimpleScan *ui, const gchar *device)
 }
 
 
+static void
+add_default_page (SimpleScan *ui)
+{
+    if (book_get_n_pages (ui->priv->book) > 0)
+        return;
+
+    /* Start with A4 white image at 72dpi */
+    // FIXME: Remember last page dimensions
+    book_append_page (ui->priv->book, 595, 842, 72, ui->priv->default_orientation);
+
+    /* Remove this page on the next scan */
+    ui->priv->book_is_placeholder = TRUE;
+}
+
+
+void new_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+new_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    book_clear (ui->priv->book);
+    add_default_page (ui);
+}
+
+
 static void
 set_document_hint (SimpleScan *ui, const gchar *document_hint)
 {
@@ -183,7 +257,7 @@ get_document_hint (SimpleScan *ui)
 static gboolean
 get_replace_pages (SimpleScan *ui)
 {
-    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check));
+    return ui->priv->book_is_placeholder || gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check));
 }
 
 
@@ -192,23 +266,28 @@ G_MODULE_EXPORT
 void
 scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    if (ui->priv->scanning) {
-        g_signal_emit (G_OBJECT (ui), signals[STOP_SCAN], 0);
-    } else {
-        gchar *device, *mode;
+    gchar *device, *mode;
 
-        device = ui_get_selected_device (ui);
-        if (device) {
-            mode = get_document_hint (ui);
-            g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
-                           FALSE, get_replace_pages (ui));
-            g_free (device);
-            g_free (mode);
-        }
+    device = ui_get_selected_device (ui);
+    if (device) {
+        mode = get_document_hint (ui);
+        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
+                       FALSE, get_replace_pages (ui));
+        g_free (device);
+        g_free (mode);
     }
 }
 
 
+void stop_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+stop_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    g_signal_emit (G_OBJECT (ui), signals[STOP_SCAN], 0);
+}
+
+
 void continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
@@ -231,6 +310,33 @@ continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
+void preferences_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+preferences_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    gtk_window_present (GTK_WINDOW (ui->priv->preferences_dialog));
+}
+
+
+gboolean preferences_dialog_delete_event_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+gboolean
+preferences_dialog_delete_event_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    return TRUE;
+}
+
+
+void preferences_dialog_response_cb (GtkWidget *widget, gint response_id, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+preferences_dialog_response_cb (GtkWidget *widget, gint response_id, SimpleScan *ui)
+{
+    gtk_widget_hide (ui->priv->preferences_dialog);
+}
+
+
 static void
 page_selected_cb (BookView *view, Page *page, SimpleScan *ui)
 {
@@ -278,9 +384,13 @@ G_MODULE_EXPORT
 void
 rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
+    Page *page;
+
     if (ui->priv->updating_page_menu)
         return;
-    page_rotate_left (book_view_get_selected (ui->priv->book_view));
+    page = book_view_get_selected (ui->priv->book_view);
+    page_rotate_left (page);
+    ui->priv->default_orientation = page_get_orientation (page);
 }
 
 
@@ -289,9 +399,13 @@ G_MODULE_EXPORT
 void
 rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
+    Page *page;
+
     if (ui->priv->updating_page_menu)
         return;
-    page_rotate_right (book_view_get_selected (ui->priv->book_view));    
+    page = book_view_get_selected (ui->priv->book_view);
+    page_rotate_right (page);
+    ui->priv->default_orientation = page_get_orientation (page);
 }
 
 
@@ -313,7 +427,21 @@ set_crop (SimpleScan *ui, const gchar *crop_name)
         page_set_no_crop (page);
         return;
     }
-    
+
+    if (strcmp (crop_name, "custom") == 0) {
+        gint width, height, crop_width, crop_height;
+
+        width = page_get_width (page);
+        height = page_get_height (page);
+
+        crop_width = (int) (width * 0.8 + 0.5);
+        crop_height = (int) (height * 0.8 + 0.5);
+        page_set_custom_crop (page, crop_width, crop_height);
+        page_move_crop (page, (width - crop_width) / 2, (height - crop_height) / 2);
+
+        return;
+    }
+
     page_set_named_crop (page, crop_name);
 }
 
@@ -462,15 +590,13 @@ draw_page (GtkPrintOperation *operation,
            SimpleScan        *ui)
 {
     cairo_t *context;
-    Book *book;
     Page *page;
     GdkPixbuf *image;
     gboolean is_landscape = FALSE;
 
     context = gtk_print_context_get_cairo_context (print_context);
    
-    book = book_view_get_book (ui->priv->book_view);
-    page = book_get_page (book, page_number);
+    page = book_get_page (ui->priv->book, page_number);
 
     /* Rotate to same aspect */
     if (gtk_print_context_get_width (print_context) > gtk_print_context_get_height (print_context))
@@ -481,8 +607,8 @@ draw_page (GtkPrintOperation *operation,
     }
    
     cairo_scale (context,
-		 gtk_print_context_get_dpi_x (print_context) / page_get_dpi (page),
-		 gtk_print_context_get_dpi_y (print_context) / page_get_dpi (page));
+                 gtk_print_context_get_dpi_x (print_context) / page_get_dpi (page),
+                 gtk_print_context_get_dpi_y (print_context) / page_get_dpi (page));
 
     image = page_get_cropped_image (page);
     gdk_cairo_set_source_pixbuf (context, image, 0, 0);
@@ -498,12 +624,6 @@ void
 email_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
     g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0);
-    GError *error = NULL;
-    g_spawn_command_line_async ("xdg-email", &error);
-    if (error) {
-        g_warning ("Unable to start email: %s", error->message);
-        g_error_free (error);
-    }
 }
 
 
@@ -515,12 +635,9 @@ print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     GtkPrintOperation *print;
     GtkPrintOperationResult result;
     GError *error = NULL;
-    Book *book;
    
-    book = book_view_get_book (ui->priv->book_view);
-
     print = gtk_print_operation_new ();
-    gtk_print_operation_set_n_pages (print, book_get_n_pages (book));
+    gtk_print_operation_set_n_pages (print, book_get_n_pages (ui->priv->book));
     g_signal_connect (print, "draw-page", G_CALLBACK (draw_page), ui);
 
     result = gtk_print_operation_run (print, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
@@ -530,24 +647,6 @@ print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
-void next_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
-G_MODULE_EXPORT
-void
-next_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
-{
-    book_view_select_next_page (ui->priv->book_view);
-}
-
-
-void prev_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
-G_MODULE_EXPORT
-void
-prev_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
-{
-    book_view_select_prev_page (ui->priv->book_view);    
-}
-
-
 static void
 load_device_cache (SimpleScan *ui)
 {
@@ -697,9 +796,11 @@ about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
                            "version", VERSION,
                            "comments", description,
                            "authors", authors,
+                           "translator-credits", _("translator-credits"),
                            "website", "https://launchpad.net/simple-scan",
                            "copyright", "Copyright (C) 2009 Canonical Ltd.",
                            "license", license,
+                           "wrap-license", TRUE,
                            NULL);
 }
 
@@ -708,6 +809,7 @@ static void
 quit (SimpleScan *ui)
 {
     char *device, *document_type;
+    gint i;
 
     save_device_cache (ui);
 
@@ -722,8 +824,16 @@ quit (SimpleScan *ui)
     g_free (document_type);
 
     gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/replace_pages",
-                          get_replace_pages (ui), NULL);
+                          gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check)), NULL);
 
+    gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_width", ui->priv->window_width, NULL);
+    gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_height", ui->priv->window_height, NULL);
+    gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/window_is_maximized", ui->priv->window_is_maximized, NULL);
+
+    for (i = 0; orientation_keys[i].key != NULL && orientation_keys[i].orientation != ui->priv->default_orientation; i++);
+    if (orientation_keys[i].key != NULL)
+        gconf_client_set_string(ui->priv->client, "/apps/simple-scan/scan_direction", orientation_keys[i].key, NULL);
+   
     g_signal_emit (G_OBJECT (ui), signals[QUIT], 0);
 }
 
@@ -737,6 +847,31 @@ quit_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
+gboolean simple_scan_window_configure_event_cb (GtkWidget *widget, GdkEventConfigure *event, SimpleScan *ui);
+G_MODULE_EXPORT
+gboolean
+simple_scan_window_configure_event_cb (GtkWidget *widget, GdkEventConfigure *event, SimpleScan *ui)
+{
+    if (!ui->priv->window_is_maximized) {
+        ui->priv->window_width = event->width;
+        ui->priv->window_height = event->height;
+    }
+
+    return FALSE;
+}
+
+
+gboolean simple_scan_window_window_state_event_cb (GtkWidget *widget, GdkEventWindowState *event, SimpleScan *ui);
+G_MODULE_EXPORT
+gboolean
+simple_scan_window_window_state_event_cb (GtkWidget *widget, GdkEventWindowState *event, SimpleScan *ui)
+{
+    if (event->changed_mask & GDK_WINDOW_STATE_MAXIMIZED)
+        ui->priv->window_is_maximized = (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED) != 0;
+    return FALSE;
+}
+
+
 gboolean window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui);
 G_MODULE_EXPORT
 gboolean
@@ -747,13 +882,29 @@ window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
 }
 
 
+static void
+page_removed_cb (Book *book, Page *page, SimpleScan *ui)
+{
+    /* Ensure always one page */
+    add_default_page (ui);
+}
+
+
+static void
+book_cleared_cb (Book *book, SimpleScan *ui)
+{
+    /* Must have been cleared for next scan */
+    ui->priv->book_is_placeholder = FALSE;
+}
+
+
 static void
 ui_load (SimpleScan *ui)
 {
     GtkBuilder *builder;
     GError *error = NULL;
     GtkCellRenderer *renderer;
-    gchar *device, *document_type;
+    gchar *device, *document_type, *scan_direction;
     gboolean replace_pages;
 
     builder = ui->priv->builder = gtk_builder_new ();
@@ -764,7 +915,8 @@ ui_load (SimpleScan *ui)
                        /* Title of dialog when cannot load required files */
                        _("Files missing"),
                        /* Description in dialog when cannot load required files */
-                       _("Please check your installation"));
+                       _("Please check your installation"),
+                       FALSE);
         exit (1);
     }
     gtk_builder_connect_signals (builder, ui);
@@ -773,17 +925,24 @@ ui_load (SimpleScan *ui)
     ui->priv->scan_button_label = GTK_WIDGET (gtk_builder_get_object (builder, "scan_button_label"));
     ui->priv->continuous_scan_button_label = GTK_WIDGET (gtk_builder_get_object (builder, "continuous_scan_button_label"));
     ui->priv->page_label = GTK_WIDGET (gtk_builder_get_object (builder, "page_label"));
-    ui->priv->actions_box = GTK_WIDGET (gtk_builder_get_object (builder, "actions_box"));
     ui->priv->device_combo = GTK_WIDGET (gtk_builder_get_object (builder, "device_combo"));
     ui->priv->device_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->device_combo));
     ui->priv->mode_combo = GTK_WIDGET (gtk_builder_get_object (builder, "mode_combo"));
     ui->priv->mode_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->mode_combo));
-    ui->priv->replace_pages_check = GTK_WIDGET (gtk_builder_get_object (builder, "replace_pages_check"));
+    ui->priv->preview_box = GTK_WIDGET (gtk_builder_get_object (builder, "preview_vbox"));
     ui->priv->preview_area = GTK_WIDGET (gtk_builder_get_object (builder, "preview_area"));
-    ui->priv->zoom_scale = GTK_WIDGET (gtk_builder_get_object (builder, "zoom_scale"));
+    ui->priv->preview_scroll = GTK_WIDGET (gtk_builder_get_object (builder, "preview_scrollbar"));
     ui->priv->page_delete_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "page_delete_menuitem"));
     ui->priv->crop_rotate_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "crop_rotate_menuitem"));
 
+    ui->priv->authorize_dialog = GTK_WIDGET (gtk_builder_get_object (builder, "authorize_dialog"));
+    ui->priv->authorize_label = GTK_WIDGET (gtk_builder_get_object (builder, "authorize_label"));
+    ui->priv->username_entry = GTK_WIDGET (gtk_builder_get_object (builder, "username_entry"));
+    ui->priv->password_entry = GTK_WIDGET (gtk_builder_get_object (builder, "password_entry"));
+   
+    ui->priv->preferences_dialog = GTK_WIDGET (gtk_builder_get_object (builder, "preferences_dialog"));
+    ui->priv->replace_pages_check = GTK_WIDGET (gtk_builder_get_object (builder, "replace_pages_check"));
+
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, TRUE);
     gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, "text", 1);
@@ -795,7 +954,7 @@ ui_load (SimpleScan *ui)
 
     /* Load previously detected scanners and select the last used one */
     load_device_cache (ui);
-    device = gconf_client_get_string(ui->priv->client, "/apps/simple-scan/selected_device", NULL);
+    device = gconf_client_get_string (ui->priv->client, "/apps/simple-scan/selected_device", NULL);
     if (device) {
         GtkTreeIter iter;
         if (find_scan_device (ui, device, &iter))
@@ -803,27 +962,49 @@ ui_load (SimpleScan *ui)
         g_free (device);
     }
     
-    document_type = gconf_client_get_string(ui->priv->client, "/apps/simple-scan/document_type", NULL);
+    document_type = gconf_client_get_string (ui->priv->client, "/apps/simple-scan/document_type", NULL);
     if (document_type) {
         set_document_hint (ui, document_type);
         g_free (document_type);
     }
 
-    replace_pages = gconf_client_get_bool (ui->priv->client, "/apps/simple-scan/replace_pages", &error);
-    if (error)
-        g_error_free (error);
-    else
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check), replace_pages);
+    replace_pages = gconf_client_get_bool (ui->priv->client, "/apps/simple-scan/replace_pages", NULL);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check), replace_pages);
     
     ui->priv->book_view = book_view_new ();
     g_signal_connect (ui->priv->book_view, "page-selected", G_CALLBACK (page_selected_cb), ui);
-    book_view_set_widget (ui->priv->book_view, ui->priv->preview_area,
-                          GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
-    gtk_range_set_adjustment (GTK_RANGE (ui->priv->zoom_scale),
-                              book_view_get_zoom_adjustment (ui->priv->book_view));
-    
-    /* Start with a reasonable size */
-    gtk_window_resize (GTK_WINDOW (ui->priv->window), 600, 400);
+    book_view_set_widgets (ui->priv->book_view,
+                           ui->priv->preview_box,
+                           ui->priv->preview_area,
+                           ui->priv->preview_scroll,
+                           GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
+
+    /* Find default scan direction */
+    scan_direction = gconf_client_get_string(ui->priv->client, "/apps/simple-scan/scan_direction", NULL);
+    if (scan_direction) {
+        gint i;
+        for (i = 0; orientation_keys[i].key != NULL && strcmp (orientation_keys[i].key, scan_direction) != 0; i++);
+        if (orientation_keys[i].key != NULL)
+            ui->priv->default_orientation = orientation_keys[i].orientation;
+    }
+
+    /* Restore window size */
+    ui->priv->window_width = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/window_width", NULL);
+    if (ui->priv->window_width <= 0)
+        ui->priv->window_width = 600;
+    ui->priv->window_height = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/window_height", NULL);
+    if (ui->priv->window_height <= 0)
+        ui->priv->window_height = 400;
+    g_debug ("Restoring window to %dx%d pixels", ui->priv->window_width, ui->priv->window_height);
+    gtk_window_set_default_size (GTK_WINDOW (ui->priv->window), ui->priv->window_width, ui->priv->window_height);
+    ui->priv->window_is_maximized = gconf_client_get_bool (ui->priv->client, "/apps/simple-scan/window_is_maximized", NULL);
+    if (ui->priv->window_is_maximized) {
+        g_debug ("Restoring window to maximized");
+        gtk_window_maximize (GTK_WINDOW (ui->priv->window));
+    }
+
+    add_default_page (ui);
+    book_view_set_book (ui->priv->book_view, ui->priv->book);
 }
 
 
@@ -834,10 +1015,10 @@ ui_new ()
 }
 
 
-void
-ui_set_book (SimpleScan *ui, Book *book)
+Book *
+ui_get_book (SimpleScan *ui)
 {
-    book_view_set_book (ui->priv->book_view, book);
+    return ui->priv->book;
 }
 
 
@@ -859,23 +1040,6 @@ void
 ui_set_scanning (SimpleScan *ui, gboolean scanning)
 {
     ui->priv->scanning = scanning;
-    if (ui->priv->scanning) {
-        gtk_label_set_label (GTK_LABEL (ui->priv->scan_button_label),
-                             /* Label on cancel scan button */
-                             _("_Cancel"));
-        gtk_label_set_label (GTK_LABEL (ui->priv->continuous_scan_button_label),
-                             /* Label on cancel scan button */
-                             _("_Cancel"));
-    }
-    else {
-        gtk_label_set_label (GTK_LABEL (ui->priv->scan_button_label),
-                             /* Label on scan button */
-                             _("_Scan"));
-        gtk_label_set_label (GTK_LABEL (ui->priv->continuous_scan_button_label),
-                             /* Label on continuous scan button */
-                             _("_Continuous Scan"));
-    }
-
     gtk_widget_set_sensitive (ui->priv->page_delete_menuitem, !scanning);
 }
 
@@ -883,24 +1047,34 @@ ui_set_scanning (SimpleScan *ui, gboolean scanning)
 void
 ui_set_have_scan (SimpleScan *ui, gboolean have_scan)
 {
-    gtk_widget_set_sensitive (ui->priv->actions_box, have_scan);
+    //FIXME: gtk_widget_set_sensitive (ui->priv->actions_box, have_scan);
 }
 
 
 void
-ui_show_error (SimpleScan *ui, const gchar *error_title, const gchar *error_text)
+ui_show_error (SimpleScan *ui, const gchar *error_title, const gchar *error_text, gboolean change_scanner_hint)
 {
     GtkWidget *dialog;
 
     dialog = gtk_message_dialog_new (GTK_WINDOW (ui->priv->window),
                                      GTK_DIALOG_MODAL,
                                      GTK_MESSAGE_WARNING,
-                                     GTK_BUTTONS_CLOSE,
+                                     GTK_BUTTONS_NONE,
                                      "%s", error_title);
+    if (change_scanner_hint)
+        gtk_dialog_add_button (GTK_DIALOG (dialog),
+                               /* Button in error dialog to open prefereces dialog and change scanner */
+                               _("Change _Scanner"),
+                               1);
+    gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CLOSE, 0);
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                               "%s", error_text);
 
-    gtk_dialog_run (GTK_DIALOG (dialog));
+    if (gtk_dialog_run (GTK_DIALOG (dialog)) == 1) {
+        gtk_widget_grab_focus (ui->priv->device_combo);
+        gtk_window_present (GTK_WINDOW (ui->priv->preferences_dialog));        
+    }
+
     gtk_widget_destroy (dialog);
 }
 
@@ -1007,10 +1181,14 @@ ui_init (SimpleScan *ui)
 {
     ui->priv = G_TYPE_INSTANCE_GET_PRIVATE (ui, SIMPLE_SCAN_TYPE, SimpleScanPrivate);
 
+    ui->priv->book = book_new ();
+    g_signal_connect (ui->priv->book, "page-removed", G_CALLBACK (page_removed_cb), ui);
+    g_signal_connect (ui->priv->book, "cleared", G_CALLBACK (book_cleared_cb), ui);
+   
     ui->priv->client = gconf_client_get_default();
     gconf_client_add_dir(ui->priv->client, "/apps/simple-scan", GCONF_CLIENT_PRELOAD_NONE, NULL);
 
     ui->priv->default_file_name = g_strdup (_("Scanned Document.pdf"));
     ui->priv->scanning = FALSE;
-    ui_load (ui);
+    ui_load (ui);   
 }
